#include <stdio.h>
#include <stdint.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <endian.h>
#include <stdlib.h>

#include <errno.h>

#ifndef BYTE_ORDER
#error "byte order not defined"
#endif

static void usage(name)
char name[];
{
	fprintf(stderr,"Usage: %s kernel.image\n",name);
	exit(1);
}

int main(argc,argv)
int argc;
char **argv;
{
	FILE *fd;
	long size, count;
	uint32_t *start,*point;

	/* start signature of squashfs */
#if BYTE_ORDER == LITTLE_ENDIAN
	uint32_t search = 0x73717368;
#else
	uint32_t search = 0x68737173;
#endif

 	if ( argc != 2 )
		usage(argv[0]);
	fd = fopen(argv[1],"r");
	if ( fd == NULL )
	{
		fprintf(stderr,"Unable to open <%s> errno: %d\n",argv[1],errno);
		usage(argv[0]);
	}
	/* get size */
    	size=fseek(fd, 0L, SEEK_END);
    	size=ftell(fd);
	fprintf(stderr,"The size of <%s> is %ld\n",argv[1],size);
    	fseek(fd, 0L, SEEK_SET);

	start=(uint32_t *)malloc(size);

	fread(start,size,1,fd);
	fclose(fd);

	for ( count=0,point=start; count < size; count+=sizeof(uint32_t),++point)
	{
		if ( *point == search )
		{
			fprintf(stderr,"Squashfs signature found at %ld\n",count);
			break;
		}
	}
	if ( count < size )
	{
		fd=fopen("kernel.raw","w");
		if ( fd == NULL )
		{
			fprintf(stderr,"Create kernel.raw: errno %d\n",errno);
			exit(99);
		}
		fwrite(start,1,count,fd);
		fclose(fd);
		fprintf(stderr,"kernel.raw created\n");
		fd=fopen("kernelsquashfs.raw","w");
		if ( fd == NULL )
		{
			fprintf(stderr,"Create kernelsquashfs.raw: errno %d\n",errno);
			exit(99);
		}
		fwrite(point,1,size-count,fd);
		fclose(fd);
		fprintf(stderr,"kernelsquashfs.raw created\n");
	}
	else
	{
		fprintf(stderr,"Strange, no squashfs signature found...\n");
		exit(99);
	}
 
	free(start);
	exit(0);
}
